@@ -6,10 +6,10 @@
 //
 //===----------------------------------------------------------------------===//
 
-#include "utils/UnitTest/Test.h"
+#include "test/UnitTest/Test.h"
 
 template <typename IntType, typename ReturnType>
-class DivTest : public __llvm_libc::testing::Test {
+class DivTest : public LIBC_NAMESPACE::testing::Test {
 public:
   using DivFunc = ReturnType(IntType, IntType);
 
